import java.io.IOException;
import java.util.Vector;

import javax.bluetooth.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;



/**
 * @author Gregor Broll, Alex De Luca
 * 
 * This midlet initiates the search for bluetooth devices and allows connecting to them (especially the gps-receiver)
 */
public class BTGPSMidlet extends MIDlet implements DiscoveryListener, CommandListener
{
	private Display display;
	private LocalDevice localDevice;						//reference to the bluetooth manager
	private DiscoveryAgent agent;							//used for device search
	private List deviceList;										//shows all discovered devices
	private Vector deviceVector;							//holds all discovered devices
	private BTDeviceSearch btDeviceSearch;
	private BTConnection btConn;
	private Form waitingForm;
		
	private static final Command CMD_DEVICE_SEARCH = new Command("Search BT Devices", Command.ITEM, 1);
	private static final Command CMD_CONNECT = new Command("Connect", Command.ITEM, 2);
	private static final Command CMD_EXIT = new Command("Exit", Command.EXIT, 1);			
	private final static Command CMD_CANCEL = new Command ("Cancel", Command.ITEM, 1);

	
	
	/**
	 * Constructor; instantiates the localdevice and the discoveryagent as well as other classes
	 */
	public BTGPSMidlet()
	{
		super();		
		
		try 
		{
			localDevice = LocalDevice.getLocalDevice();
			agent = localDevice.getDiscoveryAgent();
		} 
		catch (BluetoothStateException e) 
		{
			e.printStackTrace();
		}
		
		btDeviceSearch = null;
		btConn = null;
		deviceVector = new Vector();
		
		deviceList = new List("Discovered Bluetooth Devices", List.IMPLICIT);
		deviceList.addCommand(CMD_CONNECT);
		deviceList.addCommand(CMD_DEVICE_SEARCH);		
		deviceList.addCommand(CMD_EXIT);	
		deviceList.setCommandListener(this);
		
		waitingForm = new Form("Waiting ...");
		waitingForm.append(new Gauge("Please wait while searching for Bluetooth devices", false, Gauge.INDEFINITE, Gauge.CONTINUOUS_RUNNING));
		waitingForm.addCommand(CMD_CANCEL);
		waitingForm.setCommandListener(this);
	}

	
	/**
	 * Only shows the first interface.
	 */
	protected void startApp() throws MIDletStateChangeException
	{
		display = Display.getDisplay(this);	
		display.setCurrent(deviceList);
	}	
	
	
	/**
	 * Show all devices in the device-vector as a list.
	 */
	public void showDeviceList()
	{	
		deviceList.deleteAll();
		
		for(int i=0; i<deviceVector.size(); i++)
		{			
			try 
			{
				Object object = deviceVector.elementAt(i);
				RemoteDevice rd = (RemoteDevice)object;
				deviceList.append(rd.getFriendlyName(true), null);
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}
		}		
		
		display.setCurrent(deviceList);
	}

	
	//==============================
	//Methods of the DiscoveryListener-interface
	//==============================
	
	/**
	 * Adds discovered devices to the device-vector
	 */
	public void deviceDiscovered(RemoteDevice rd, DeviceClass arg1) 
	{
		deviceVector.addElement(rd);
	}
	
	
	/**
	 * Called after the inquiry is finished; shows discovered devices
	 */
	public void inquiryCompleted(int type) 
	{
		Alert info;
		
		if (type == DiscoveryListener.INQUIRY_COMPLETED)
			info = new Alert("", "Inquiry terminated - " + deviceVector.size(), null, AlertType.INFO);
		else
			info = new Alert("", "Inquiry terminated with error - " + deviceVector.size(), null, AlertType.INFO);	
		
		info.setTimeout(Alert.FOREVER);
		display.setCurrent(info, deviceList);
		
		showDeviceList();
	}

	
	/**
	 * not used
	 */
	public void servicesDiscovered(int arg0, ServiceRecord[] arg1) 
	{
		
	}


	/**
	 * not used
	 */
	public void serviceSearchCompleted(int arg0, int arg1) 
	{
		
	}

	
	/**
	 * 
	 */
	protected void pauseApp()
	{

	}

	
	/**
	 *  
	 */
	protected void destroyApp(boolean arg0) throws MIDletStateChangeException
	{
		notifyDestroyed();
	}


	/**
	 * Command-handler
	 */
	public void commandAction(Command cmd, Displayable arg1) 
	{		
		//initiate a new search for bluetooth devices
		if(cmd == CMD_DEVICE_SEARCH)
		{			
			deviceVector.removeAllElements();
			display.setCurrent(waitingForm);
			
			btDeviceSearch = new BTDeviceSearch(this, agent);			
			btDeviceSearch.run();
		}
		
		if(cmd == CMD_EXIT)
		{
			try 
			{
				destroyApp(true);
			} 
			catch (MIDletStateChangeException e) 
			{
				e.printStackTrace();
			}
		}
		
		//cancels a search for bluetooth devices
		if(cmd == CMD_CANCEL)
		{
			btDeviceSearch.cancelInquiry(this);
			
			Alert info = new Alert("", "Search for Bluetooth devices canceled", null, AlertType.INFO);			
			info.setTimeout(Alert.FOREVER);
			display.setCurrent(info, deviceList);
			
			showDeviceList();
		}
		
		//connect to a selected bluetooth device
		if(cmd == CMD_CONNECT)
		{
			RemoteDevice rd = (RemoteDevice)deviceVector.elementAt(deviceList.getSelectedIndex());

			try 
			{				
				btConn = new BTConnection(this, rd.getFriendlyName(true), rd.getBluetoothAddress());
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}	
		}
	}
}
